<?php

namespace Snog\Movies\Cron;

use XF\Db\Schema\Alter;
use XF\Util\File;

class Convert
{
	public static function Process()
	{
		$app = \XF::app();
		$db = \XF::db();
		$sm = $db->getSchemaManager();
		$columns = $sm->getTableColumnDefinitions('xf_thread');

		if (array_key_exists('tmdb_id', $columns))
		{
			$movies = $db->fetchAll('SELECT * FROM xf_thread WHERE tmdb_id > "" AND tmdb_id > 0 ORDER BY post_date DESC limit 6');
		}
		else
		{
			$movies = '';
		}

		if (!$movies)
		{
			// REMOVE OLD XF1 FIELDS
			$sm->alterTable('xf_thread', function (Alter $table)
			{
				$table->dropColumns([
					'tmdb_id',
					'tmdb_title',
					'tmdb_plot',
					'tmdb_image',
					'tmdb_genres',
					'tmdb_director',
					'tmdb_cast',
					'tmdb_release',
					'tmdb_tagline',
					'tmdb_runtime',
					'tmdb_rating',
					'tmdb_votes',
					'tmdb_thread'
				]);
			});

			// DELETE OLD BB CODES
			$db->query('DELETE FROM xf_bb_code WHERE addon_id = "Snog/Movies"');

			// DISABLE CRON TASK
			$db->query('UPDATE xf_cron_entry SET active = 0 WHERE entry_id = "snogMoviesConvert"');
		}
		else
		{
			foreach ($movies as $thread)
			{
				$comment = '';
				$post = $app->finder('XF:Post')->where('post_id', $thread['first_post_id'])->fetchOne();

				if (isset($post->message))
				{
					$message = $post->message;

					if (!strstr($message, '[/CONTAINER]'))
					{
						$comment = $message;
					}
					else
					{
						$split = explode("[/media]", $message);
						if (!isset($split[1]))
						{
							if ($thread['tmdb_plot'] > '' && $thread['tmdb_plot'] !== NULL)
							{
								$split2 = explode($thread['tmdb_plot'], $message);
								if (!stristr($split2[1], 'media') && $split2[1])
								{
									$comment = $split2[1];
								}
							}
						}
						else
						{
							$comment = $split[1];
						}
					}
				}

				if($comment == '[/PLOT][/MOVIE][/CONTAINER]') $comment = '';

				try
				{
					$response = \XF::app()->http()->client()->get("http://api.themoviedb.org/3/movie/" . $thread['tmdb_id'] . "?api_key=" . $app->options()->tmdbthreads_apikey . "&language=" . $app->options()->tmdbthreads_language . "&append_to_response=casts,trailers");
				}
				catch(\GuzzleHttp\Exception\RequestException $e)
				{
					continue;
				}

				$movie = $response->json();

				if(!isset($movie['title']))
				{
					$filename = 'data/movies/conversionerror.log';
					$somecontent = 'No movie title supplied by TMDB - Thread: ' . $thread['thread_id'] . ' TMDB ID:' . $thread['tmdb_id'] . "\r\n";
					$handle = fopen($filename, 'a');
					fwrite($handle, $somecontent);
					fclose($handle);
					continue;
				}

				$trailer = '';
				foreach ($movie['trailers']['youtube'] as $source)
				{
					$trailer = $source['source'];
				}

				$posterpath = $thread['tmdb_image'];

				if (isset($movie['poster_path']))
				{
					$tempPath = FILE::getTempDir() . $posterpath;

					$posterpath = $movie['poster_path'];
					$src = "http://image.tmdb.org/t/p/w92" . $posterpath;
					$path = 'data://movies/SmallPosters' . $posterpath;
					self::getimage($src, $path, $tempPath);

					$src = "http://image.tmdb.org/t/p/w185" . $posterpath;
					$path = 'data://movies/LargePosters' . $posterpath;
					self::getimage($src, $path, $tempPath);
				}

				$cast = $thread['tmdb_cast'];
				if ($cast === NULL) $cast = '';

				$newMovie = $app->em()->create('Snog\Movies:Movie');
				$newMovie->thread_id = $thread['thread_id'];
				$newMovie->tmdb_id = $thread['tmdb_id'];
				$newMovie->tmdb_title = $thread['tmdb_title'];
				$newMovie->tmdb_plot = $thread['tmdb_plot'];
				$newMovie->tmdb_image = $posterpath;
				$newMovie->tmdb_genres = $thread['tmdb_genres'];
				$newMovie->tmdb_director = $thread['tmdb_director'];
				$newMovie->tmdb_cast = $cast;
				$newMovie->tmdb_release = $thread['tmdb_release'];
				$newMovie->tmdb_tagline = $thread['tmdb_tagline'];
				$newMovie->tmdb_runtime = $thread['tmdb_runtime'];
				$newMovie->tmdb_rating = $thread['tmdb_rating'];
				$newMovie->tmdb_votes = $thread['tmdb_votes'];
				$newMovie->tmdb_trailer = $trailer;
				if (!$app->options()->tmdbthreads_force_comments) $newMovie->comment = $comment;
				$newMovie->save(false, false);

				if (!$posterpath) $posterpath = 'no-poster.jpg';

				if (isset($post->message))
				{
					$message = "[IMG]" . self::getImageUrl(($posterpath ? $posterpath : '/no-poster.jpg')) . '[/IMG]' . "\r\n\r\n";
					$message .= "[B]" . \XF::phrase('title') . ":[/B] " . $thread['tmdb_title'] . "\r\n\r\n";
					if ($thread['tmdb_tagline']) $message .= "[B]" . \XF::phrase('snog_movies_tagline') . ":[/B] " . $thread['tmdb_tagline'] . "\r\n\r\n";
					if ($thread['tmdb_genres']) $message .= "[B]" . \XF::phrase('snog_movies_genre') . ":[/B] " . $thread['tmdb_genres'] . "\r\n\r\n";
					if ($thread['tmdb_director']) $message .= "[B]" . \XF::phrase('snog_movies_director') . ":[/B] " . $thread['tmdb_director'] . "\r\n\r\n";
					if ($cast) $message .= "[B]" . \XF::phrase('snog_movies_cast') . ":[/B] " . $cast . "\r\n\r\n";
					if ($thread['tmdb_release']) $message .= "[B]" . \XF::phrase('snog_movies_release') . ":[/B] " . $thread['tmdb_release'] . "\r\n\r\n";
					if ($thread['tmdb_runtime']) $message .= "[B]" . \XF::phrase('snog_movies_runtime') . ":[/B] " . $thread['tmdb_runtime'] . "\r\n\r\n";
					if ($thread['tmdb_plot']) $message .= "[B]" . \XF::phrase('snog_movies_plot') . ":[/B] " . $thread['tmdb_plot'] . "\r\n\r\n";
					$message .= "[MEDIA=youtube]" . $trailer . "[/MEDIA]" . "\r\n\r\n";
					if (!$app->options()->tmdbthreads_force_comments) $message .= $comment;
					$post->message = $message;
					$post->save(false, false);

					// RETAINS FIRST POST AS SECOND POST WHEN REGULAR MOVIE INFO IS NOT PRESENT IN FIRST POST
					if ($comment && $app->options()->tmdbthreads_force_comments)
					{
						$newpost = $app->em()->create('XF:Post');
						$newpost->thread_id = $thread['thread_id'];
						$newpost->user_id = $thread['user_id'];
						$newpost->username = $thread['username'];
						$newpost->post_date = $thread['post_date'];
						$newpost->message = $comment;
						$newpost->ip_id = $post->ip_id;
						$newpost->position = 1;
						$newpost->last_edit_date = 0;
						$newpost->save();

						$postorder = $app->finder('XF:Post')->where('thread_id', $thread['thread_id'])->order('post_date', 'ASC')->fetch();
						$order = 1;
						foreach ($postorder as $changeorder)
						{
							if ($changeorder->post_id <> $thread['first_post_id'])
							{
								$changeorder->position = $order;
								$changeorder->save();
								$order = $order + 1;
							}
						}
					}
				}

				$db->query('UPDATE xf_thread SET tmdb_id = 0 WHERE thread_id = "' . $thread['thread_id'] . '"');

			}
		}
	}

	public static function getImageUrl($posterpath, $canonical = true)
	{
		$app = \XF::app();
		return $app->applyExternalDataUrl("movies/LargePosters{$posterpath}", $canonical);
	}

	protected static function getimage($src, $path, $tempPath)
	{
		try
		{
			$response = \XF::app()->http()->client()->get($src);
		}
		catch(\GuzzleHttp\Exception\RequestException $e)
		{
			return;
		}

		if (file_exists($tempPath)) unlink($tempPath);
		file_put_contents($tempPath, $response->getBody());
		File::copyFileToAbstractedPath($tempPath, $path);
		unlink($tempPath);
	}
}