<?php

namespace Snog\Movies\Pub\Controller;

use XF\Pub\Controller\AbstractController;
use XF\Mvc\ParameterBag;
use XF\Util\File;

class Movies extends AbstractController
{
	public function actionRate(ParameterBag $params)
	{
		$visitor = \XF::visitor();
		$movie = $this->finder('Snog\Movies:Movie')->where('thread_id', $params->thread_id)->fetchOne();
		$childMovie = $this->finder('Snog\Movies:Movie')->where('tmdb_id', $movie->tmdb_id)->where('thread_id', '>', $params->thread_id)->fetch();
		$userRating = array();
		if ($visitor->user_id) $userRating = $this->finder('Snog\Movies:Rating')->where('thread_id', $movie->thread_id)->where('user_id', $visitor->user_id)->fetchOne();

		if ($this->isPost())
		{
			$thisRating = $this->filter('rating', 'uint');

			if (!isset($userRating->user_id))
			{
				$newRating = $this->em()->create('Snog\Movies:Rating');
				$newRating->thread_id = $movie->thread_id;
				$newRating->user_id = $visitor->user_id;
				$newRating->rating = $thisRating;
				$newRating->save();
			}
			else
			{
				$userRating->rating = $thisRating;
				$userRating->save();
			}

			$finder = $this->finder('Snog\Movies:Rating');
			$ratings = $finder->where('thread_id', $movie->thread_id)->order('user_id', 'ASC')->fetch();
			$ratingcount = $finder->total();
			$total = 0;

			foreach ($ratings as $rating)
			{
				$total = $total + $rating->rating;
			}

			$newRating = round(($total / $ratingcount), 2);
			$movie->tmdb_rating = $newRating;
			$movie->tmdb_votes = $ratingcount;
			$movie->save();

			// ADD RATING TO CROSSLINKED THREADS
			if ($childMovie)
			{
				foreach ($childMovie as $child)
				{
					$child->tmdb_rating = $newRating;
					$child->tmdb_votes = $ratingcount;
					$child->save();
				}
			}

			$thread = $this->assertViewableThread($params->thread_id);
			return $this->redirect($this->buildLink('threads', $thread));
		}
		else
		{
			$currentRating = 0;
			if (isset($userRating->rating)) $currentRating = $userRating->rating;
			$viewParams = ['movie' => $movie, 'currentRating' => $currentRating];
			return $this->view('Snog:Movies\Movie', 'snog_movies_rate', $viewParams);
		}
	}

	public function actionEdit(ParameterBag $params)
	{
		$thread = $this->assertViewableThread($params->thread_id);
		$post = $this->assertViewablePost($thread->first_post_id, ['Thread.Prefix']);
		if (!$post->canEdit($error)) return $this->noPermission($error);
		$movie = $this->finder('Snog\Movies:Movie')->where('thread_id', $params->thread_id)->fetchOne();
		$porta = $this->filter('porta', 'bool');

		if ($this->isPost())
		{
			$input = $this->filter([
				'title' => 'str',
				'tmdb_title' => 'str',
				'tmdb_tagline' => 'str',
				'tmdb_genres' => 'str',
				'tmdb_director' => 'str',
				'tmdb_cast' => 'str',
				'tmdb_release' => 'str',
				'tmdb_runtime' => 'uint',
				'tmdb_plot' => 'str'
			]);

			$releaseExploded = explode('-', $input['tmdb_release']);
			if (!isset($releaseExploded[0]) || strlen($releaseExploded[0]) !== 4) return $this->error('Release date must be formatted YYYY-MM-DD');

			$url = $this->filter('tmdb_trailer', 'str');
			$trailer = '';
			$threadChanges = [];

			if (stristr($url, 'youtube'))
			{
				$mediaRepo = $this->repository('XF:BbCodeMediaSite');
				$sites = $mediaRepo->findActiveMediaSites()->fetch();
				$match = $mediaRepo->urlMatchesMediaSiteList($url, $sites);
				if ($match) $trailer = $match['media_id'];
			}
			else
			{
				$trailer = $url;
			}

			$comment = $this->plugin('XF:Editor')->fromInput('message');

			$message = "[img]" . $this->getImageUrl(($movie->tmdb_image ? $movie->tmdb_image : 'no-poster.jpg')) . "[/img]" . "\r\n\r\n";
			$message .= "[B]" . \XF::phrase('title') . ":[/B] " . $input['tmdb_title'] . "\r\n\r\n";
			if ($input['tmdb_tagline']) $message .= "[B]" . \XF::phrase('snog_movies_tagline') . ":[/B] " . $input['tmdb_tagline'] . "\r\n\r\n";
			if ($input['tmdb_genres']) $message .= "[B]" . \XF::phrase('snog_movies_genre') . ":[/B] " . $input['tmdb_genres'] . "\r\n\r\n";
			if ($input['tmdb_director']) $message .= "[B]" . \XF::phrase('snog_movies_director') . ":[/B] " . $input['tmdb_director'] . "\r\n\r\n";
			if ($input['tmdb_cast']) $message .= "[B]" . \XF::phrase('snog_movies_cast') . ":[/B] " . $input['tmdb_cast'] . "\r\n\r\n";
			if ($input['tmdb_release']) $message .= "[B]" . \XF::phrase('snog_movies_release') . ":[/B] " . $input['tmdb_release'] . "\r\n\r\n";
			if ($input['tmdb_runtime']) $message .= "[B]" . \XF::phrase('snog_movies_runtime') . ":[/B] " . $input['tmdb_runtime'] . "\r\n\r\n";
			if ($input['tmdb_plot']) $message .= "[B]" . \XF::phrase('snog_movies_plot') . ":[/B] " . $input['tmdb_plot'] . "\r\n\r\n";
			if ($trailer) $message .= "[MEDIA=youtube]" . $trailer . "[/MEDIA]" . "\r\n\r\n";
			if ($comment && !$this->options()->tmdbthreads_force_comments) $message .= $comment;

			$post->last_edit_date = 0;
			$post->message = $message;
			$post->save();

			if ($input['tmdb_title'] !== $thread->title)
			{
				$year = '';
				if ($this->options()->tmdbthreads_useyear)
				{
					if ($input['tmdb_release']) $releaseExploded = explode('-', $input['tmdb_release']);
					if (isset($releaseExploded[0])) $year = ' (' . $releaseExploded[0] . ')';
				}
				$thread->title = $input['tmdb_title'] . $year;
				$thread->save();
				$threadChanges = ['title' => 1];
			}

			$movie->tmdb_title = $input['tmdb_title'];
			$movie->tmdb_plot = $input['tmdb_plot'];
			$movie->tmdb_genres = $input['tmdb_genres'];
			$movie->tmdb_director = $input['tmdb_director'];
			$movie->tmdb_cast = $input['tmdb_cast'];
			$movie->tmdb_release = $input['tmdb_release'];
			$movie->tmdb_tagline = $input['tmdb_tagline'];
			$movie->tmdb_runtime = $input['tmdb_runtime'];
			$movie->tmdb_trailer = $trailer;
			if (!$this->options()->tmdbthreads_force_comments) $movie->comment = $comment;
			if ($this->options()->tmdbthreads_force_comments) $movie->comment = '';
			$movie->save(false, false);

			if ($this->filter('_xfWithData', 'bool'))
			{
				$viewParams = ['post' => $post, 'thread' => $thread];

				if (!$porta)
				{
					$reply = $this->view('XF:Post\EditNewPost', 'post_edit_new_post', $viewParams);
				}
				else
				{
					$reply = $this->view('XF:Post\EditNewPost', 'snog_movies_XenPorta', $viewParams);
				}

				$reply->setJsonParams([
					'message' => \XF::phrase('your_changes_have_been_saved'),
					'threadChanges' => ($porta ? null : $threadChanges)
				]);
				return $reply;
			}
			else
			{
				return $this->redirect($this->buildLink('posts', $post));
			}
		}
		else
		{
			$forum = $post->Thread->Forum;
			if ($forum->canUploadAndManageAttachments())
			{
				$attachmentRepo = $this->repository('XF:Attachment');
				$attachmentData = $attachmentRepo->getEditorData('post', $post);
			}
			else
			{
				$attachmentData = null;
			}

			$prefix = $thread->Prefix;
			$prefixes = $forum->getUsablePrefixes($prefix);

			$viewParams = [
				'movie' => $movie,
				'porta' => $porta,
				'post' => $post,
				'thread' => $thread,
				'forum' => $forum,
				'prefixes' => $prefixes,
				'attachmentData' => $attachmentData,
				'quickEdit' => $this->filter('_xfWithData', 'bool')
			];
			return $this->view('XF:Post\Edit', 'snog_movies_movie_edit', $viewParams);
		}
	}

	protected function assertViewableThread($threadId, array $extraWith = [])
	{
		$visitor = \XF::visitor();
		$extraWith[] = 'Forum';
		$extraWith[] = 'Forum.Node';
		$extraWith[] = 'Forum.Node.Permissions|' . $visitor->permission_combination_id;

		if ($visitor->user_id)
		{
			$extraWith[] = 'Read|' . $visitor->user_id;
			$extraWith[] = 'Forum.Read|' . $visitor->user_id;
		}

		/** @var \XF\Entity\Thread $thread */
		$thread = $this->em()->find('XF:Thread', $threadId, $extraWith);
		if (!$thread) throw $this->exception($this->notFound(\XF::phrase('requested_thread_not_found')));
		if (!$thread->canView($error)) throw $this->exception($this->noPermission($error));
		$this->plugin('XF:Node')->applyNodeContext($thread->Forum->Node);
		$this->setContentKey('thread-' . $thread->thread_id);
		return $thread;
	}

	protected function assertViewablePost($postId, array $extraWith = [])
	{
		$visitor = \XF::visitor();
		$extraWith[] = 'Thread';
		$extraWith[] = 'Thread.Forum';
		$extraWith[] = 'Thread.Forum.Node';
		$extraWith[] = 'Thread.Forum.Node.Permissions|' . $visitor->permission_combination_id;

		/** @var \XF\Entity\Post $post */
		$post = $this->em()->find('XF:Post', $postId, $extraWith);
		if (!$post) throw $this->exception($this->notFound(\XF::phrase('requested_post_not_found')));
		if (!$post->canView($error)) throw $this->exception($this->noPermission($error));
		$this->plugin('XF:Node')->applyNodeContext($post->Thread->Forum->Node);
		return $post;
	}

	public function actionPoster(ParameterBag $params)
	{
		if ($this->isPost())
		{
			$visitor = \XF::visitor();
			if ($visitor['user_id'] < 1 || (!$visitor->is_moderator && !$visitor->is_admin)) throw $this->exception($this->noPermission());

			$movie = $this->finder('Snog\Movies:Movie')->where('thread_id', $params->thread_id)->fetchOne();
			$oldposter = $movie->tmdb_image;
			$posterpath = $this->filter('posterpath', 'str');

			$thread = $this->assertViewableThread($params->thread_id);
			$post = $this->assertViewablePost($thread->first_post_id, ['Thread.Prefix']);

			$tempPath = FILE::getTempDir() . $posterpath;

			$src = "http://image.tmdb.org/t/p/w92" . $posterpath;
			$path = 'data://movies/SmallPosters' . $posterpath;
			$this->getimage($src, $path, $tempPath);

			$src = "http://image.tmdb.org/t/p/w185" . $posterpath;
			$path = 'data://movies/LargePosters' . $posterpath;
			$this->getimage($src, $path, $tempPath);

			$movie->tmdb_image = $posterpath;
			$movie->save(false, false);

			$message = "[IMG]" . $this->getImageUrl(($posterpath ? $posterpath : '/no-poster.jpg')) . '[/IMG]' . "\r\n\r\n";
			$message .= "[B]" . \XF::phrase('title') . ":[/B] " . $movie->tmdb_title . "\r\n\r\n";
			if ($movie->tmdb_tagline) $message .= "[B]" . \XF::phrase('snog_movies_tagline') . ":[/B] " . $movie->tmdb_tagline . "\r\n\r\n";
			if ($movie->tmdb_genres) $message .= "[B]" . \XF::phrase('snog_movies_genre') . ":[/B] " . $movie->tmdb_genres . "\r\n\r\n";
			if ($movie->tmdb_director) $message .= "[B]" . \XF::phrase('snog_movies_director') . ":[/B] " . $movie->tmdb_director . "\r\n\r\n";
			if ($movie->tmdb_cast) $message .= "[B]" . \XF::phrase('snog_movies_cast') . ":[/B] " . $movie->tmdb_cast . "\r\n\r\n";
			if ($movie->tmdb_release) $message .= "[B]" . \XF::phrase('snog_movies_release') . ":[/B] " . $movie->tmdb_release . "\r\n\r\n";
			if ($movie->tmdb_runtime) $message .= "[B]" . \XF::phrase('snog_movies_runtime') . ":[/B] " . $movie->tmdb_runtime . "\r\n\r\n";
			if ($movie->tmdb_plot) $message .= "[B]" . \XF::phrase('snog_movies_plot') . ":[/B] " . $movie->tmdb_plot . "\r\n\r\n";
			if ($movie->tmdb_trailer) $message .= "[MEDIA=youtube]" . $movie->tmdb_trailer . "[/MEDIA]" . "\r\n\r\n";
			if ($movie->comment && !$this->options()->tmdbthreads_force_comments) $message .= $movie->comment;

			$post->message = $message;
			$post->last_edit_date = 0;
			$post->save();

			if ($oldposter)
			{
				$path = sprintf('data://movies/LargePosters%s', $oldposter);
				File::deleteFromAbstractedPath($path);

				$path = sprintf('data://movies/SmallPosters%s', $oldposter);
				File::deleteFromAbstractedPath($path);
			}

			return $this->redirect($this->buildLink('posts', $post));
		}
		else
		{
			$visitor = \XF::visitor();
			if ($visitor['user_id'] < 1 || (!$visitor->is_moderator && !$visitor->is_admin)) throw $this->exception($this->noPermission());
			$movie = $this->finder('Snog\Movies:Movie')->where('thread_id', $params->thread_id)->fetchOne();
			$newPoster = false;
			$posterpath = '';

			try
			{
				$response = \XF::app()->http()->client()->get("http://api.themoviedb.org/3/movie/" . $movie->tmdb_id . "?api_key=" . $this->options()->tmdbthreads_apikey . "&language=" . $this->options()->tmdbthreads_language);
			}
			catch (\GuzzleHttp\Exception\RequestException $e)
			{
				if (null !== $e->getResponse())
				{
					$error = 'TMDb Error ' . $e->getResponse()->getStatusCode();
					$error .= ': ' . $e->getResponse()->getReasonPhrase();
				}
				else
				{
					$error = $e->getMessage();
				}

				return $this->error($error);
			}

			$movieinfo = \GuzzleHttp\json_decode($response->getBody(), true);

			if (!isset($movieinfo['id'])) return $this->error('Movie Information Not Returned from TMDB.');

			if (isset($movieinfo['poster_path'])) $posterpath = $movieinfo['poster_path'];
			if ($posterpath !== $movie->tmdb_image) $newPoster = true;

			$viewParams = ['movie' => $movie, 'newposter' => $newPoster, 'posterpath' => $posterpath];
			return $this->view('Snog\Movies:Movies', 'snog_movies_new_poster', $viewParams);
		}
	}

	public function actionAddinfo(ParameterBag $params)
	{
		if ($this->isPost())
		{
			$visitor = \XF::visitor();
			if ($visitor->user_id < 1 || (!$visitor->is_moderator && !$visitor->is_admin)) throw $this->exception($this->noPermission());

			$thread = $this->assertViewableThread($params->thread_id);
			$post = $this->assertViewablePost($thread->first_post_id, ['Thread.Prefix']);
			$title = $this->filter('tmdb', 'str');
			$changeTitle = $this->filter('changetitle', 'uint');
			$tmdbid = '';

			if (!$title) return $this->error(\XF::phrase('snog_movies_error_no_movie'));

			if (stristr($title, "themoviedb"))
			{
				// preg_match_all USED FOR FUTURE API PARAMETER CAPTURING
				preg_match_all('/\d+/', $title, $matches);

				if (!empty($matches))
				{
					$tmdbid = $matches[0][0];
				}
				else
				{
					$tmdbid = '';
				}
			}
			else if (is_numeric($title))
			{
				if (intval($title) == 0)
				{
					$tmdbid = '';
				}
				else
				{
					$tmdbid = $title;
				}
			}

			if (!$tmdbid) return $this->error(\XF::phrase('snog_movies_error_not_valid'));

			$exists = $this->finder('Snog\Movies:Movie')->where('tmdb_id', $tmdbid)->fetchOne();
			$comment = $post->message;

			// MOVIE ALREADY EXISTS
			if (isset($exists->tmdb_id))
			{
				return $this->error(\XF::phrase('snog_movies_error_posted'));
			}

			try
			{
				$response = \XF::app()->http()->client()->get("http://api.themoviedb.org/3/movie/" . $tmdbid . "?api_key=" . $this->options()->tmdbthreads_apikey . "&language=" . $this->options()->tmdbthreads_language . "&append_to_response=casts,trailers");
			}
			catch (\GuzzleHttp\Exception\RequestException $e)
			{
				if (null !== $e->getResponse())
				{
					$error = 'TMDb Error ' . $e->getResponse()->getStatusCode();
					$error .= ': ' . $e->getResponse()->getReasonPhrase();
				}
				else
				{
					$error = $e->getMessage();
				}

				return $this->error($error);
			}

			$movie = \GuzzleHttp\json_decode($response->getBody(), true);

			$genres = '';
			foreach ($movie['genres'] as $genre)
			{
				if ($genres) $genres .= ', ';
				$genres .= $genre['name'];
			}

			$directors = '';
			foreach ($movie['casts']['crew'] as $director)
			{
				if ($director['job'] == 'Director')
				{
					if ($directors) $directors .= ', ';
					$directors .= str_replace(',', '', $director['name']);
				}
			}

			$cast = '';
			foreach ($movie['casts']['cast'] as $member)
			{
				if ($cast) $cast .= ', ';
				$cast .= str_replace(',', '', $member['name']);
			}

			$trailer = '';
			foreach ($movie['trailers']['youtube'] as $source)
			{
				$trailer = $source['source'];
			}

			$runtime = '';
			if ($movie['runtime']) $runtime = $movie['runtime'];
			$tagline = '';
			if ($movie['tagline']) $tagline = html_entity_decode($movie['tagline']);
			$releasedate = '';
			if ($movie['release_date']) $releasedate = $movie['release_date'];
			$movietitle = '';
			if ($movie['title']) $movietitle = html_entity_decode($movie['title']);
			$plot = '';
			if ($movie['overview']) $plot = html_entity_decode($movie['overview']);

			$posterpath = '';
			if (isset($movie['poster_path']))
			{
				$posterpath = $movie['poster_path'];
				$tempPath = FILE::getTempDir() . $posterpath;

				$src = "http://image.tmdb.org/t/p/w92" . $posterpath;
				$path = 'data://movies/SmallPosters' . $posterpath;
				$this->getimage($src, $path, $tempPath);

				$src = "http://image.tmdb.org/t/p/w185" . $posterpath;
				$path = 'data://movies/LargePosters' . $posterpath;
				$this->getimage($src, $path, $tempPath);
			}

			$message = "[IMG]" . $this->getImageUrl(($posterpath ? $posterpath : '/no-poster.jpg')) . '[/IMG]' . "\r\n\r\n";
			$message .= "[B]" . \XF::phrase('title') . ":[/B] " . $movietitle . "\r\n\r\n";
			if ($tagline) $message .= "[B]" . \XF::phrase('snog_movies_tagline') . ":[/B] " . $tagline . "\r\n\r\n";
			if ($genres) $message .= "[B]" . \XF::phrase('snog_movies_genre') . ":[/B] " . $genres . "\r\n\r\n";
			if ($directors) $message .= "[B]" . \XF::phrase('snog_movies_director') . ":[/B] " . $directors . "\r\n\r\n";
			if ($cast) $message .= "[B]" . \XF::phrase('snog_movies_cast') . ":[/B] " . $cast . "\r\n\r\n";
			if ($releasedate) $message .= "[B]" . \XF::phrase('snog_movies_release') . ":[/B] " . $releasedate . "\r\n\r\n";
			if ($runtime) $message .= "[B]" . \XF::phrase('snog_movies_runtime') . ":[/B] " . $runtime . "\r\n\r\n";
			if ($plot) $message .= "[B]" . \XF::phrase('snog_movies_plot') . ":[/B] " . $plot . "\r\n\r\n";
			if ($trailer) $message .= "[MEDIA=youtube]" . $trailer . "[/MEDIA]" . "\r\n\r\n";
			if (!$this->options()->tmdbthreads_force_comments) $message .= $comment;

			if ($changeTitle)
			{
				$year = '';
				$releaseExploded = [];
				if ($this->options()->tmdbthreads_useyear)
				{
					if ($releasedate) $releaseExploded = explode('-', $releasedate);
					if (isset($releaseExploded[0])) $year = ' (' . $releaseExploded[0] . ')';
				}
				$thread->title = $movietitle . $year;
				$thread->save();
			}

			$post->message = $message;
			$post->last_edit_date = 0;
			$post->save();

			if ($comment && $this->options()->tmdbthreads_force_comments)
			{
				$newfirstpost = false;
				$newlastpost = false;

				$newpost = $this->em()->create('XF:Post');
				$newpost->thread_id = $thread->thread_id;
				$newpost->user_id = $thread->user_id;
				$newpost->username = $thread->username;
				$newpost->post_date = $thread->post_date;
				$newpost->message = $comment;
				$newpost->ip_id = $post->ip_id;
				$newpost->position = 1;
				$newpost->last_edit_date = 0;
				$newpost->save();
				$newpostId = $newpost->getEntityId();

				if ($thread->first_post_id > 0 && $thread->first_post_id <> $post->post_id) $newfirstpost = true;
				if ($thread->first_post_id == $thread->last_post_id) $newlastpost = true;

				if ($newfirstpost) $thread->first_post_id = $newpostId;
				if ($newlastpost)
				{
					$thread->last_post_date = $thread->post_date;
					$thread->last_post_id = $newpostId;
					$thread->last_post_user_id = $thread->user_id;
					$thread->last_post_username = $thread->username;
				}
				if ($thread->isChanged('first_post_id') || $thread->isChanged('last_post_id')) $thread->save();

				$postorder = $this->finder('XF:Post')->where('thread_id', $thread->thread_id)->order('post_date', 'ASC')->fetch();
				$order = 1;
				foreach ($postorder as $changeorder)
				{
					if ($changeorder->post_id <> $thread->first_post_id)
					{
						$changeorder->position = $order;
						$changeorder->save();
						$order = $order + 1;
					}
				}
			}

			$movie = $this->em()->create('Snog\Movies:Movie');
			$movie->thread_id = $thread->thread_id;
			$movie->tmdb_id = $tmdbid;
			$movie->tmdb_title = $movietitle;
			$movie->tmdb_plot = $plot;
			$movie->tmdb_image = $movie['poster_path'];
			$movie->tmdb_genres = $genres;
			$movie->tmdb_director = $directors;
			$movie->tmdb_cast = $cast;
			$movie->tmdb_release = $releasedate;
			$movie->tmdb_tagline = $tagline;
			$movie->tmdb_runtime = $runtime;
			$movie->tmdb_trailer = $trailer;
			if (!$this->options()->tmdbthreads_force_comments) $movie->comment = $comment;
			$movie->save(false, false);

			return $this->redirect($this->buildLink('threads', $thread));
		}
		else
		{
			$visitor = \XF::visitor();
			if ($visitor->user_id < 1 || (!$visitor->is_moderator && !$visitor->is_admin)) throw $this->exception($this->noPermission());
			$thread = $this->finder('XF:Thread')->where('thread_id', $params->thread_id)->fetchOne();
			$viewParams = ['thread' => $thread];
			return $this->view('Snog\Movies:Movies', 'snog_movies_add_info', $viewParams);
		}
	}

	public function getImageUrl($posterpath, $canonical = true)
	{
		return $this->app->applyExternalDataUrl("movies/LargePosters{$posterpath}", $canonical);
	}

	protected function getimage($src, $path, $tempPath)
	{
		try
		{
			$response = \XF::app()->http()->client()->get($src);
		} catch (\GuzzleHttp\Exception\RequestException $e)
		{
			$error = $e->getMessage();
			return $this->error($error);
		}

		if (file_exists($tempPath)) unlink($tempPath);
		file_put_contents($tempPath, $response->getBody());
		File::copyFileToAbstractedPath($tempPath, $path);
		unlink($tempPath);
	}
}