<?php

namespace Snog\Movies;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;
use XF\Util\File;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;
	
	public function checkRequirements(&$errors = [], &$warnings = [])
	{
		$versionId = $this->addOn->version_id;
	
		if($versionId && $versionId < '28')
		{
			$errors[] = 'Upgrades can only be to the XF 1.x TMDb Movie Thread Starter version 2.1.13 or later';
			return $errors;
		}

		return $errors;
	}
	
	public function installStep1()
	{
		$sm = $this->schemaManager();
	
		$sm->createTable('xf_snog_movies_thread', function(Create $table)
		{
			$table->addColumn('thread_id', 'INT', 10);
			$table->addColumn('tmdb_id', 'INT', 10)->setDefault(0);
			$table->addColumn('tmdb_title', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_plot', 'TEXT')->setDefault(NULL);
			$table->addColumn('tmdb_image', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_genres', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_director', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_cast', 'TEXT')->setDefault(NULL);
			$table->addColumn('tmdb_release', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_tagline', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_runtime', 'INT', 10)->setDefault(0);
			$table->addColumn('tmdb_rating')->type('decimal', '3,2')->setDefault(0);
			$table->addColumn('tmdb_votes', 'INT', 10)->setDefault(0);
			$table->addColumn('tmdb_trailer', 'VARCHAR', 255)->setDefault('');
			$table->addColumn('comment', 'TEXT')->setDefault(NULL);
			$table->addPrimaryKey('thread_id');
			$table->addKey('tmdb_id');
		});
	}
	
	public function installStep2()
	{
		$sm = $this->schemaManager();
	
		$sm->createTable('xf_snog_movies_forum', function(Create $table)
		{
			$table->addColumn('node_id', 'INT', 10);
			$table->addColumn('tmdb_genre', 'BLOB')->nullable();
			$table->addPrimaryKey('node_id');
		});
	}
	
	public function installStep3()
	{
		$sm = $this->schemaManager();
	
		$sm->createTable('xf_snog_movies_ratings', function(Create $table)
		{
			$table->addColumn('rating_id', 'INT', 10)->autoIncrement();
			$table->addColumn('thread_id', 'INT', 10)->setDefault(0);
			$table->addColumn('user_id', 'INT', 10)->setDefault(0);
			$table->addColumn('rating', 'TINYINT', 1)->setDefault(0);
			$table->addPrimaryKey('rating_id');
			$table->addKey('thread_id');
			$table->addKey('user_id');
		});
	}
	
	public function installStep4()
	{
		$src = 'src/addons/Snog/Movies/defaultdata';
		$this->copyContents($src);
	}
	
	public function upgrade2000070Step1()
	{
		$sm = $this->schemaManager();
		
		$sm->createTable('xf_snog_movies_thread', function(Create $table)
		{
			$table->addColumn('thread_id', 'INT', 10);
			$table->addColumn('tmdb_id', 'INT', 10)->setDefault(0);
			$table->addColumn('tmdb_title', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_plot', 'TEXT')->setDefault(NULL);
			$table->addColumn('tmdb_image', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_genres', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_director', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_cast', 'TEXT')->setDefault(NULL);
			$table->addColumn('tmdb_release', 'VARCHAR', 150)->setDefault('');
			$table->addColumn('tmdb_tagline', 'TEXT')->setDefault(NULL);
			$table->addColumn('tmdb_runtime', 'INT', 10)->setDefault(0);
			$table->addColumn('tmdb_rating')->type('decimal', '3,2')->setDefault(0);
			$table->addColumn('tmdb_votes', 'INT', 10)->setDefault(0);
			$table->addColumn('tmdb_trailer', 'VARCHAR', 255)->setDefault('');
			$table->addColumn('comment', 'TEXT')->setDefault(NULL);
			$table->addPrimaryKey('thread_id');
			$table->addKey('tmdb_id');
		});
	}
	
	public function upgrade2000070Step2()
	{
		$db = $this->db();
		$sm = $this->schemaManager();
	
		$sm->createTable('xf_snog_movies_forum', function(Create $table)
		{
			$table->addColumn('node_id', 'INT', 10);
			$table->addColumn('tmdb_genre', 'BLOB')->nullable();
			$table->addPrimaryKey('node_id');
		});
		
		// COPY OLD FORUM FIELD TO NEW TABLE
		$tmpData = $db->fetchAll("SELECT node_id, tmdb_genre FROM xf_forum WHERE tmdb_genre > ''");
		
		foreach($tmpData as $data)
		{
			$genres = explode(',', $data['tmdb_genre']);
			
			$db->insert('xf_snog_movies_forum', [
					'node_id' => $data['node_id'],
					'tmdb_genre' => serialize($genres)
			]);
		}
		
		// DROP OLD COLUMN
		$sm->alterTable('xf_forum', function (Alter $table)
		{
			$table->dropColumns(['tmdb_genre']);
		});
	}
	
	public function upgrade2000070Step3()
	{
		$sm = $this->schemaManager();
	
		$sm->createTable('xf_snog_movies_ratings', function(Create $table)
		{
			$table->addColumn('rating_id', 'INT', 10)->autoIncrement();
			$table->addColumn('thread_id', 'INT', 10)->setDefault(0);
			$table->addColumn('user_id', 'INT', 10)->setDefault(0);
			$table->addColumn('rating', 'TINYINT', 1)->setDefault(0);
			$table->addPrimaryKey('rating_id');
			$table->addKey('thread_id');
			$table->addKey('user_id');
		});
	}
	
	public function upgrade2000070Step4()
	{
		$src = 'src/addons/Snog/Movies/defaultdata';
		$this->copyContents($src);
	}

	public function upgrade2000170Step1()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_snog_movies_thread', function (Alter $table)
		{
			$table->changeColumn('tmdb_tagline','TEXT')->setDefault(NULL);
		});

	}
	
	public function uninstallStep1()
	{
		$sm = $this->schemaManager();
		$sm->dropTable('xf_snog_movies_thread');
		$sm->dropTable('xf_snog_movies_ratings');
		$sm->dropTable('xf_snog_movies_forum');
	}
	
	public function copyContents($src, $sub = false)
	{
		if($sub) $basePath = str_ireplace('src/addons/Snog/Movies/defaultdata/', '', $src);
		$dir = opendir($src);
	
		while(false !== ( $file = readdir($dir)) )
		{
			if(($file != '.') && ($file != '..'))
			{
				if(is_dir($src . '/' . $file) )
				{
					$newSrc = $src . '/' . $file;
					$this->copyContents($newSrc, true);
				}else{
					$oldPath = $src . '/'. $file;
						
					if($sub)
					{
						$newFile = $basePath . '/' . $file;
					}else{
						$newFile = $file;
					}
						
					$newPath = sprintf('data://movies/%s', $newFile);
					File::copyFileToAbstractedPath($oldPath, $newPath);
				}
			}
		}
		
		closedir($dir);
	}
}