<?php

namespace Snog\Movies\XF\Admin\Controller;

use XF\Mvc\FormAction;

class Forum extends XFCP_Forum
{
	protected function nodeAddEdit(\XF\Entity\Node $node)
	{
		$reply = parent::nodeAddEdit($node);

		if (in_array($node->node_id, $this->options()->tmdbthreads_forum) && $reply instanceof \XF\Mvc\Reply\View)
		{
			try
			{
				$response = \XF::app()->http()->client()->get("http://api.themoviedb.org/3/genre/movie/list?api_key=" . $this->options()->tmdbthreads_apikey . "&language=" . $this->options()->tmdbthreads_language);
			}
			catch(\GuzzleHttp\Exception\RequestException $e)
			{
				if(null !== $e->getResponse())
				{
					$error = 'TMDb Error ' . $e->getResponse()->getStatusCode();
					$error .= ': ' . $e->getResponse()->getReasonPhrase();
				}
				else
				{
					$error = $e->getMessage();
				}

				return $this->error($error);
			}

			$codes = \GuzzleHttp\json_decode($response->getBody(), true);

			$availableGenres['All'] = 'All';

			foreach ($codes['genres'] as $genre)
			{
				$availableGenres[$genre['name']] = $genre['name'];
			}

			$reply->setParams(['availableGenres' => $availableGenres]);
		}

		return $reply;
	}

	protected function saveTypeData(FormAction $form, \XF\Entity\Node $node, \XF\Entity\AbstractNode $data)
	{
		if (in_array($node->node_id, $this->options()->tmdbthreads_forum) && $node->node_id !== NULL)
		{
			$newForum = false;
			$movieForum = $this->finder('Snog\Movies:Forum')->where('node_id', $node->node_id)->fetchOne();

			if (!isset($movieForum->node_id))
			{
				$movieForum = $this->em()->create('Snog\Movies:Forum');
				$newForum = true;
			}

			$input = $this->filter(['available_genres' => 'array']);
			if ($newForum) $movieForum->node_id = $node->node_id;
			$movieForum->tmdb_genre = $input['available_genres'];
			$movieForum->save();
		}
		/** @noinspection PhpVoidFunctionResultUsedInspection */
		return parent::saveTypeData($form, $node, $data);
	}
}