<?php

namespace Snog\Movies\XF\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Util\File;

class Forum extends XFCP_Forum
{
	public function actionPostThread(ParameterBag $params)
	{
		if ($this->isPost())
		{
			if (!$params->node_id && !$params->node_name) return parent::actionPostThread($params);
			$forum = $this->assertViewableForum($params->node_id ?: $params->node_name, ['DraftThreads|' . \XF::visitor()->user_id]);
			if (!in_array($forum->node_id, $this->options()->tmdbthreads_forum)) return parent::actionPostThread($params);

			$originalNode = $forum->node_id;
			$title = $this->filter('title', 'str');
			if ($this->options()->tmdbthreads_mix && !stristr($title, "themoviedb") && !is_numeric($title)) return parent::actionPostThread($params);

			if (!$forum->canCreateThread($error)) return $this->noPermission($error);
			$switches = $this->filter(['inline-mode' => 'bool', 'more-options' => 'bool']);
			if ($switches['more-options']) return parent::actionPostThread($params);

			if ($switches['more-options']) $switches['inline-mode'] = false;
			$thread = null;
			$post = null;
			$allowed = ($this->options()->tmdbthreads_use_genres ? false : true);
			$crosslink = ($this->options()->tmdbthreads_use_genres && $this->options()->tmdbthreads_crosslink ? true : false);
			if (\XF::visitor()->isShownCaptcha() && !$this->app->captcha()->isValid()) return $this->error(\XF::phrase('did_not_complete_the_captcha_verification_properly'));

			$comment = $this->plugin('XF:Editor')->fromInput('message');

			if (stristr($title, "themoviedb.org/tv/")) return $this->error(\XF::phrase('snog_movies_error_tv_id'));

			if (stristr($title, "themoviedb"))
			{
				// preg_match_all USED FOR FUTURE API PARAMETER CAPTURING
				preg_match_all('/\d+/', $title, $matches);

				if (!empty($matches))
				{
					$tmdbid = $matches[0][0];
				}
				else
				{
					$tmdbid = '';
				}
			}
			else if (is_numeric($title))
			{
				if (intval($title) == 0)
				{
					$tmdbid = '';
				}
				else
				{
					$tmdbid = $title;
				}
			}
			else
			{
				$tmdbid = '';
			}

			if (!$tmdbid) return $this->error(\XF::phrase('snog_movies_error_not_valid'));

			$exists = $this->finder('Snog\Movies:Movie')->where('tmdb_id', $tmdbid)->fetchOne();

			// MOVIE ALREADY EXISTS - IF COMMENTS MADE POST TO EXISTING THREAD
			if (isset($exists->tmdb_id) && $comment)
			{
				$thread = $this->finder('XF:Thread')->where('thread_id', $exists->thread_id)->fetchOne();
				$replier = $this->service('XF:Thread\Replier', $thread);
				$replier->setMessage($comment);
				if ($forum->canUploadAndManageAttachments()) $replier->setAttachmentHash($this->filter('attachment_hash', 'str'));
				$post = $replier->save();
				return $this->redirect($this->plugin('XF:Thread')->getPostLink($post));
			}

			// MOVIE ALREADY EXISTS - NO COMMENTS - SEND TO EXISTING THREAD
			if (isset($exists->tmdb_id))
			{
				$thread = $this->finder('XF:Thread')->where('thread_id', $exists->thread_id)->fetchOne();
				return $this->redirect($this->buildLink('threads', $thread));
			}

			// GET THE MOVIE
			try
			{
				$response = \XF::app()->http()->client()->get("http://api.themoviedb.org/3/movie/" . $tmdbid . "?api_key=" . $this->options()->tmdbthreads_apikey . "&language=" . $this->options()->tmdbthreads_language . "&append_to_response=casts,trailers");
			}
			catch (\GuzzleHttp\Exception\RequestException $e)
			{
				if (null !== $e->getResponse())
				{
					$error = 'TMDb Error ' . $e->getResponse()->getStatusCode();
					$error .= ': ' . $e->getResponse()->getReasonPhrase();
				}
				else
				{
					$error = $e->getMessage();
				}

				return $this->error($error);
			}

			$movie = \GuzzleHttp\json_decode($response->getBody(), true);

			$genres = '';
			foreach ($movie['genres'] as $genre)
			{
				if ($genres) $genres .= ', ';
				$genres .= $genre['name'];

				// CHECK IF GENRE IS ALLOWED IN THIS FORUM
				if (!$allowed)
				{
					if (in_array($genre['name'], $forum->Movieforum->tmdb_genre)) $allowed = true;
				}
			}

			// GENRE NOT ALLOWED? CHECK ALL MOVIE FORUMS
			if (!$allowed)
			{
				$movieForums = $this->finder('Snog\Movies:Forum')->fetch();
				$checkGenres = explode(',', $genres);

				foreach ($movieForums as $genreForum)
				{
					foreach ($checkGenres as $genre)
					{
						if (in_array(trim($genre), $genreForum->tmdb_genre))
						{
							$allowed = true;
							$forum = $this->finder('XF:Forum')->where('node_id', $genreForum->node_id)->fetchOne();
							break;
						}
					}
				}

				// LAST CHANCE CHECK FOR CATCHALL FORUM
				if (!$allowed)
				{
					foreach ($movieForums as $genreForum)
					{
						if (in_array('All', $genreForum->tmdb_genre))
						{
							$allowed = true;
							$forum = $this->finder('XF:Forum')->where('node_id', $genreForum->node_id)->fetchOne();
							break;
						}
					}
				}

				if (!$allowed) return $this->error(\XF::phrase('snog_movies_error_genre'));
			}

			$directors = '';
			foreach ($movie['casts']['crew'] as $director)
			{
				if ($director['job'] == 'Director')
				{
					if ($directors) $directors .= ', ';
					$directors .= str_replace(',', '', $director['name']);
				}
			}

			$cast = '';
			foreach ($movie['casts']['cast'] as $member)
			{
				if ($cast) $cast .= ', ';
				$cast .= str_replace(',', '', $member['name']);
			}

			$trailer = '';
			foreach ($movie['trailers']['youtube'] as $source)
			{
				$trailer = $source['source'];
			}

			$runtime = 0;
			if ($movie['runtime']) $runtime = $movie['runtime'];
			$tagline = html_entity_decode($movie['tagline']);
			$releasedate = $movie['release_date'];
			$movietitle = html_entity_decode($movie['title']);
			$posterpath = '';

			$plot = html_entity_decode($movie['overview']);

			if (isset($movie['poster_path']))
			{
				$posterpath = $movie['poster_path'];
				$tempPath = FILE::getTempDir() . $posterpath;

				$posterpath = $movie['poster_path'];
				$src = "http://image.tmdb.org/t/p/w92" . $posterpath;
				$path = 'data://movies/SmallPosters' . $posterpath;
				$this->getMovieimage($src, $path, $tempPath);

				$src = "http://image.tmdb.org/t/p/w185" . $posterpath;
				$path = 'data://movies/LargePosters' . $posterpath;
				$this->getMovieimage($src, $path, $tempPath);
			}

			$message = "[IMG]" . $this->getMovieImageUrl(($posterpath ? $posterpath : '/no-poster.jpg')) . '[/IMG]' . "\r\n\r\n";
			$message .= "[B]" . \XF::phrase('title') . ":[/B] " . $movietitle . "\r\n\r\n";
			if ($tagline) $message .= "[B]" . \XF::phrase('snog_movies_tagline') . ":[/B] " . $tagline . "\r\n\r\n";
			if ($genres) $message .= "[B]" . \XF::phrase('snog_movies_genre') . ":[/B] " . $genres . "\r\n\r\n";
			if ($directors) $message .= "[B]" . \XF::phrase('snog_movies_director') . ":[/B] " . $directors . "\r\n\r\n";
			if ($cast) $message .= "[B]" . \XF::phrase('snog_movies_cast') . ":[/B] " . $cast . "\r\n\r\n";
			if ($releasedate) $message .= "[B]" . \XF::phrase('snog_movies_release') . ":[/B] " . $releasedate . "\r\n\r\n";
			if ($runtime) $message .= "[B]" . \XF::phrase('snog_movies_runtime') . ":[/B] " . $runtime . "\r\n\r\n";
			if ($plot) $message .= "[B]" . \XF::phrase('snog_movies_plot') . ":[/B] " . $plot . "\r\n\r\n";
			if ($trailer) $message .= "[MEDIA=youtube]" . $trailer . "[/MEDIA]" . "\r\n\r\n";
			if (!$this->options()->tmdbthreads_force_comments) $message .= $comment;

			$title = $movietitle;
			$releaseExploded = [];
			if ($this->options()->tmdbthreads_useyear)
			{
				$year = '';
				if ($releasedate) $releaseExploded = explode('-', $releasedate);
				if (isset($releaseExploded[0])) $year = ' (' . $releaseExploded[0] . ')';
				$title .= $year;
			}

			$creator = $this->service('XF:Thread\Creator', $forum);
			$creator->setContent($title, $message);

			$prefixId = $this->filter('prefix_id', 'uint');
			if ($prefixId && $forum->isPrefixUsable($prefixId)) $creator->setPrefix($prefixId);
			if ($forum->canEditTags()) $creator->setTags($this->filter('tags', 'str'));
			if (!$this->options()->tmdbthreads_force_comments && $forum->canUploadAndManageAttachments()) $creator->setAttachmentHash($this->filter('attachment_hash', 'str'));

			$setOptions = $this->filter('_xfSet', 'array-bool');
			if ($setOptions)
			{
				$thread = $creator->getThread();

				if (isset($setOptions['discussion_open']) && $thread->canLockUnlock())
				{
					$creator->setDiscussionOpen($this->filter('discussion_open', 'bool'));
				}
				if (isset($setOptions['sticky']) && $thread->canStickUnstick())
				{
					$creator->setSticky($this->filter('sticky', 'bool'));
				}
			}

			$customFields = $this->filter('custom_fields', 'array');
			$creator->setCustomFields($customFields);
			$creator->checkForSpam();

			/** @var \XF\Service\Thread\Creator $errors */
			if (!$creator->validate($errors)) return $this->error($errors);
			$this->assertNotFlooding('post');

			/** @var \XF\Entity\Thread $thread */
			$thread = $creator->save();

			$movie = $this->em()->create('Snog\Movies:Movie');
			$movie->thread_id = $thread->thread_id;
			$movie->tmdb_id = $tmdbid;
			$movie->tmdb_title = $title;
			$movie->tmdb_plot = $plot;
			$movie->tmdb_image = $posterpath;
			$movie->tmdb_genres = $genres;
			$movie->tmdb_director = $directors;
			$movie->tmdb_cast = $cast;
			$movie->tmdb_release = $releasedate;
			$movie->tmdb_tagline = $tagline;
			$movie->tmdb_runtime = $runtime;
			$movie->tmdb_trailer = $trailer;
			if ($comment && !$this->options()->tmdbthreads_force_comments) $movie->comment = $comment;
			$movie->save(false, false);

			if ($comment && $this->options()->tmdbthreads_force_comments)
			{
				$replier = $this->service('XF:Thread\Replier', $thread);
				$replier->setMessage($comment);
				if ($forum->canUploadAndManageAttachments()) $replier->setAttachmentHash($this->filter('attachment_hash', 'str'));
				$replier->save();
			}

			// CREATE REDIRECT THREADS IN APPROPRIATE FORUMS
			if ($crosslink)
			{
				$movieForums = $this->finder('Snog\Movies:Forum')->fetch();
				$checkGenres = explode(',', $genres);
				$data = $thread->toArray(false);
				$noLink[] = $data['node_id'];
				unset($data['thread_id'], $data['node_id']);
				$data['first_post_id'] = 0;
				$data['discussion_type'] = 'redirect';

				foreach ($movieForums as $genreForum)
				{
					foreach ($checkGenres as $genre)
					{
						if (in_array(trim($genre), $genreForum->tmdb_genre) && !in_array($genreForum->node_id, $noLink))
						{
							$noLink[] = $genreForum->node_id;
							$data['node_id'] = $genreForum->node_id;

							$crosslink = $this->em()->create('XF:Thread');
							$crosslink->bulkSet($data);
							$crosslink->save();
							$crosslinkThreadId = $crosslink->getEntityId();

							$redirect = $this->em()->create('XF:ThreadRedirect');
							$redirect->thread_id = $crosslinkThreadId;
							$redirect->target_url = $this->app()->router('public')->buildLink('nopath:threads', $thread);
							$redirect->redirect_key = "thread-{$thread->thread_id}-{$thread->node_id}-";
							$redirect->expiry_date = 0;
							$redirect->save();

							$movie = $this->em()->create('Snog\Movies:Movie');
							$movie->thread_id = $crosslinkThreadId;
							$movie->tmdb_id = $tmdbid;
							$movie->tmdb_title = $title;
							$movie->tmdb_plot = $plot;
							$movie->tmdb_image = $posterpath;
							$movie->tmdb_genres = $genres;
							$movie->tmdb_director = $directors;
							$movie->tmdb_cast = $cast;
							$movie->tmdb_release = $releasedate;
							$movie->tmdb_tagline = $tagline;
							$movie->tmdb_runtime = $runtime;
							$movie->tmdb_trailer = $trailer;
							$movie->save(false, false);
						}
					}
				}
			}

			// MOVED HERE FOR COMPATIBLITY ISSUES WITH XenPorta REDIRECTING IN THE FUNCTION
			/** @var \XF\Service\Thread\Creator $creator */
			$this->finalizeThreadCreate($creator);

			if ($switches['inline-mode'] && $forum->node_id == $originalNode)
			{
				$viewParams = ['thread' => $thread, 'forum' => $forum];
				return $this->view('XF:Forum\ThreadItem', 'thread_list_item', $viewParams);
			}
			else if (!$thread->canView())
			{
				return $this->redirect($this->buildLink('forums', $forum, ['pending_approval' => 1]));
			}
			else
			{
				return $this->redirect($this->buildLink('threads', $thread));
			}
		}

		return parent::actionPostThread($params);
	}

	protected function getAvailableForumSorts(\XF\Entity\Forum $forum)
	{
		$sorts = parent::getAvailableForumSorts($forum);
		$sorts['Movie.tmdb_director'] = 'Movie.tmdb_director';
		$sorts['Movie.tmdb_release'] = 'Movie.tmdb_release';
		$sorts['Movie.tmdb_rating'] = 'Movie.tmdb_rating';
		$sorts['Movie.tmdb_genres'] = 'Movie.tmdb_genres';
		return $sorts;
	}

	protected function getForumFilterInput(\XF\Entity\Forum $forum)
	{
		$filters = parent::getForumFilterInput($forum);
		$genre = $this->filter('genre', 'str');
		$director = $this->filter('director', 'str');
		$title = $this->filter('movie_title', 'str');
		if ($genre) $filters['genre'] = $genre;
		if ($director) $filters['director'] = $director;
		if ($title) $filters['movie_title'] = $title;
		return $filters;
	}

	protected function applyForumFilters(\XF\Entity\Forum $forum, \XF\Finder\Thread $threadFinder, array $filters)
	{
		if (!empty($filters['genre'])) $threadFinder->where('Movie.tmdb_genres', 'LIKE', '%' . $filters['genre'] . '%');
		if (!empty($filters['director'])) $threadFinder->where('Movie.tmdb_director', 'LIKE', '%' . $filters['director'] . '%');
		if (!empty($filters['movie_title'])) $threadFinder->where('Movie.tmdb_title', 'LIKE', '%' . $filters['movie_title'] . '%');
		parent::applyForumFilters($forum, $threadFinder, $filters);
	}

	protected function getMovieimage($src, $path, $tempPath)
	{
		try
		{
			$response = \XF::app()->http()->client()->get($src);
		}
		catch (\GuzzleHttp\Exception\RequestException $e)
		{
			$error = $e->getMessage();
			return $this->error($error);
		}

		if (file_exists($tempPath)) unlink($tempPath);
		file_put_contents($tempPath, $response->getBody());
		File::copyFileToAbstractedPath($tempPath, $path);
		unlink($tempPath);
	}

	public function getMovieImageUrl($posterpath, $canonical = true)
	{
		return $this->app->applyExternalDataUrl("movies/LargePosters{$posterpath}", $canonical);
	}
}
